package app.mar.game;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.ArrayList;

import app.mar.utils.files.JSONAssetsManager;
import min3d.core.Object3d;
import min3d.parser.IParser;
import min3d.parser.Parser;
import min3d.vos.Number3d;

/**
 * Created by ptitcois on 19/08/16.
 */
public class Resource implements Serializable {

    protected String mName;
    protected transient Object3d  m3DModel = null;
    protected Number3d mPosition = new Number3d(0,0,0);
    protected Number3d mRotation = new Number3d(0,0,0);
    protected Number3d mScale = new Number3d(1,1,1);
    protected String mComment = "null";
    protected boolean isLoaded = false;
    protected boolean mIsDisplay = false;
    protected int   mStage;
    protected String mType = ""; //3D Image Audio Video
    protected String mTitle= ""; //3D Image Audio Video
    private  static JSONObject mRessourcesList = null;
    public static ArrayList<String> mNotFound = new ArrayList<>();
    protected String mFile = null;

    public String getFile() {
        if(mFile!=null)
            return mFile;
        return mName;
    }

    public boolean isFile() {
        return mFile!=null;
    }

    public static void initResourceList(Activity act)
    {
        if(mRessourcesList!=null) return;
        Game.logv( "Chargement de la liste des ressources...");
        JSONObject obj = JSONAssetsManager.load(act, "ressources.res");
        if(obj==null)
        {
            Game.logw("Attention il n'existe pas de fichier pour la liste des ressrouecs");
            return;
        }
        mRessourcesList=obj;
    }

    public static JSONObject getRessourceDataFromList(String name)
    {
        try {
            if(mRessourcesList!=null)
                return mRessourcesList.getJSONObject(name);
        } catch (JSONException e) {
            Game.logw("Impossiblle de trouver la ressource '"+name+"' dans la liste des ressources");
        }
        return null;
    }

    public static void logNotFound()
    {
        Log.e("LoadingRes", "Liste des ressources non-trouvées:");
        for(int i=0; i<mNotFound.size(); i++)
            Log.e("LoadingRes", "\t'"+mNotFound.get(i)+"'");
    }

    public static void setNotFound(String name)
    {
        for(int i=0; i< mNotFound.size(); i++)
            if(mNotFound.get(i).compareTo(name)==0) return;
        mNotFound.add(name);
    }

    protected void loadResourceData(Activity act)
    {
        initResourceList(act);
        Game.logv( "--- Chargement de la ressource '"+mName+"' --- ");
        JSONObject obj = JSONAssetsManager.load(act, mName+"_res");
        if(obj==null) obj = JSONAssetsManager.load(act, mName+".res");
        if(obj==null) obj = getRessourceDataFromList(mName);
        if(obj == null)
        {
            Game.loge("\tLa ressource '"+mName+"' est introuvable.");
            setNotFound(mName);
            Game.fail();
            return;
        }

        //mandatory
        try {
            mType = obj.getString("type");
        } catch(JSONException e)
        {
            Game.loge("\tLa ressource '"+mName+"' n'a pas d'attribut 'type' valide (String)");
            e.printStackTrace();
        }

        try {
            mTitle = obj.getString("title");
        } catch (JSONException e) {
        }

        try {
            mComment=obj.getString("comment");
        } catch (JSONException e) {
        }
        try {
            mStage = obj.getInt("stage");
            isLoaded=true;
        } catch (JSONException e) {
        }

        try{
            mFile = obj.getString("file");
        }catch (JSONException e)
        {
            mFile=null;
        }


        try {
            mPosition.x = (float) obj.getJSONArray("position").getDouble(0);
            mPosition.y = (float) obj.getJSONArray("position").getDouble(1);
            mPosition.z = (float) obj.getJSONArray("position").getDouble(2);


            mRotation.x = (float) obj.getJSONArray("rotation").getDouble(0);
            mRotation.y = (float) obj.getJSONArray("rotation").getDouble(1);
            mRotation.z = (float) obj.getJSONArray("rotation").getDouble(2);


            mScale.x = (float) obj.getJSONArray("scale").getDouble(0);
            mScale.y = (float) obj.getJSONArray("scale").getDouble(1);
            mScale.z = (float) obj.getJSONArray("scale").getDouble(2);
        }catch (JSONException e) {
        }

        try {
            mIsDisplay=(boolean) obj.getBoolean("display");
        } catch (JSONException e) {
            mIsDisplay=false;
        }
    }

    public boolean hasFile(){
        return mFile!=null;
    }

    public void loadModel(Activity context)
    {
        get3DModel(context);
    }

    public Number3d getPosition() {
        return mPosition;
    }

    public Number3d getRotation() {
        return mRotation;
    }

    public Number3d getScale() {
        return mScale;
    }

    public Resource(String name, Activity act)
    {
        mName=name;
        loadResourceData(act);
    }


    public String getName() {
        return mName;
    }

    public String getType() {
        return mType;
    }

    public String getTitle() {
        return mTitle;
    }


    public String getComment() {
        return mComment;
    }

    public int getStage() {
        return mStage;
    }

    public  JSONObject getJson()
    {
        JSONObject jo = new JSONObject();
        try {
            jo.put("name", mName);
            jo.put("stage", mStage);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jo;
    }



    public Object3d get3DModel(Activity context)
    {
        if(m3DModel==null)
        {
            IParser myParser = Parser.createParser(context, Parser.Type.OBJ,  mName+"_obj",true);
            myParser.parse();

            m3DModel = myParser.getParsedObject();
            m3DModel.position().x = mPosition.x; m3DModel.position().y = mPosition.y; m3DModel.position().z = mPosition.z;
            m3DModel.rotation().x = mRotation.x; m3DModel.rotation().y = mRotation.y; m3DModel.rotation().z = mRotation.z;
            m3DModel.rotation().x+=45;
            //m3DModel.rotation().y+=-180;

            //m3DModel.rotation().y=90;
            m3DModel.scale().x = mScale.x; m3DModel.scale().y = mScale.y; m3DModel.scale().z = mScale.z;
        }
        return m3DModel;

    }



    public void delete3DModel()
    {
        m3DModel = null;
    }

    public boolean is3DModelLoaded()
    {
        return m3DModel!=null;
    }

    public void scale3DModel(float factor)
    {
        if(m3DModel!=null)
            m3DModel.scale().x = m3DModel.scale().y = m3DModel.scale().z = factor;
    }

    public String toString()
    {
        return mTitle;
    }

    public boolean isAudio()
    {
        return mType.toLowerCase().compareTo("audio")==0;
    }

    public boolean isVideo()
    {
        return mType.toLowerCase().compareTo("video")==0;
    }
    public boolean is3D()
    {
        return mType.toLowerCase().compareTo("3d")==0;
    }

    public boolean isImage()
    {
        return mType.toLowerCase().compareTo("image")==0;
    }

    public boolean isDisplay() {
        return mIsDisplay;
    }
}
